/*******************************************************************************
				Neoinput
				========

		Copyright (C) 2010 Paul Kocialkowski

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"

#include "server.h"
#include "input_write.h"

void print_version(void)
{
	printf("%s_server %s, %s\n", SOFTWARE, VERSION, DESCRIPTION);
	printf("%s\n", COPYRIGHT);
}

void print_help(void)
{
	print_version();
	printf("\nUsage: %s_server [OPTION]...\n", SOFTWARE);
	printf("\nOptions:\n");
	printf("  --help        Shows this help message.\n");
	printf("  --version     Shows the program version.\n");
	printf("  -a [ADDRESS]  Specify the IP address to listen on. Default is %s.\n", DEFAULT_IP);
	printf("  -p [PORT]     Specify the port to listen on.\n");
	printf("  -d [FILE]     Specify the device to inject data on. Default is uinput.\n");
	printf("  -s [FACTOR]   Specify the speed factor of the input device.\n");
}

int main(int argc, char **argv)
{
	int i, j;

	int server_port=0;
	char *server_ip=NULL;

	char append=0;

	/* Check args */

	for(i=1 ; i < argc ; i++)
	{
		if(argv[i][0] == '-')
		{
			if(argv[i][1] == '-')
			{
				if(strcmp(argv[i], "--help") == 0)
				{
					print_help();
					exit(0);
				}
				else if(strcmp(argv[i], "--version") == 0)
				{
					print_version();
					exit(0);
				}
				else
				{
					printf("Error: Unknown argument '%s'.\n", argv[i]);
					exit(1);
				}
			}
			else
			{
				for(j=1 ; j < strlen(argv[i]) ; j++)
				{
					if(append)
					{
						printf("Error: missing value for argument '%c'.\n", append);
						exit(1);
					}
					
					switch(argv[i][j])
					{
						case 'a':
							append='a';
						break;

						case 'd':
							append='d';
						break;

						case 'p':
							append='p';
						break;

						case 's':
							append='s';
						break;

						default:
							printf("Error: Unknown argument '%c'.\n", argv[i][j]);
						break;
					}
				}
			}
		}
		else
		{
			if(append)
			{
				if(append == 's' && speed_factor == 1)
					speed_factor=atof(argv[i]);
				else if(append == 'd' && input_device == NULL)
					input_device=argv[i];
				else if(append == 'p' && server_port == 0)
					server_port=atoi(argv[i]);
				else if(append == 'a' && server_ip == NULL)
					server_ip=argv[i];
				else
				{
					printf("Error: Argument '%c' redifinition.\n", append);
					exit(1);
				}

				append=0;
			}
			else
			{
				printf("Error: Unknown argument '%s'.\n", argv[i]);
				exit(1);
			}
		}
	}

	if(append)
	{
		printf("Error: missing value for argument '%c'.\n", append);
		exit(1);
	}

	if(server_port == 0)
	{
		printf("No server port specified, using	: DEFAULT_PORT (%d).\n", DEFAULT_PORT);
		server_port=DEFAULT_PORT;
	}
	if(server_ip == NULL)
	{
		printf("No server ip specified, using	: INADDR_ANY.\n");
		server_ip=NULL;
	}
	if(input_device == NULL)
	{
		printf("No input device specified, using: uinput.\n");
		input_device=NULL;
	}

	printf("speed_factor = %lf\n", speed_factor);

	/* Open the server */
	server_open(server_ip, server_port);
	
	/* Accept connections */
	server_accept();

	/* Close the server */
	server_close();
}
