/*******************************************************************************
				Neoinput
				========

		Copyright (C) 2010 Paul Kocialkowski

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include <linux/input.h>
#include "input.h"

#include "config.h"
#include "input_write.h"
#include "server.h"

/* Sockets files descriptors */
int sock_fd, sock_client_fd;

/* Struct to store the server address */
struct sockaddr_in server_addr;

/* Struct to store the server address */
struct sockaddr_in client_addr;

void server_open(char *server_ip, int server_port)
{
	/* Start to open a socket */
	sock_fd = socket(AF_INET, SOCK_STREAM, 0);

	/* Check if the socket is ok */
	if(sock_fd == -1)
	{
		perror("Error in socket()");
		exit(1);
	}
	else
		printf("socket(): OK...\n");

	/* Server address family */
	server_addr.sin_family=AF_INET;

	/* Server port */
	server_addr.sin_port=htons(server_port);

	/* Server address */
	if(server_ip == NULL)
		server_addr.sin_addr.s_addr=INADDR_ANY;
	else
		server_addr.sin_addr.s_addr=inet_addr(server_ip);

	/* Set the Rest of the struct to 0 */

	memset(&(server_addr.sin_zero), 0, 8);

	/* Bind and check if bind is ok */ 
	if(bind(sock_fd, (struct sockaddr *)&server_addr, sizeof(struct sockaddr)) == -1)
	{
		perror("Error in bind()");
		exit(1);
	}
	else
		printf("bind(): OK...\n");
 
	/* Listen (10 connection max on queue) and cgeck if listen is ok */
	if(listen(sock_fd, 10) == -1)
	{
		perror("Error in listen()");
		exit(1);
	}
	else
		printf("listen(): OK...\n");
}

void server_accept(void)
{
	int sockaddr_in_size;
	int input_type;

	/* Get struct sockaddr_in size */
	sockaddr_in_size=sizeof(struct sockaddr_in);

	while(sock_client_fd=accept(sock_fd, (struct sockaddr *)&client_addr, &sockaddr_in_size) )
	{
		if(sock_client_fd == -1)
		{
			perror("Error in accept()");
		}
		else
			printf("accept(): OK...\n");

		printf("New Connection\n");

		input_type=INPUT_TYPE_ERROR;

		/* Get the input type */
		input_type=server_get_input_type();

		if((input_type != INPUT_TYPE_TOUCH && 
		input_type != INPUT_TYPE_TOUCH_ROTATE && 
		input_type != INPUT_TYPE_ACCEL && 
		input_type != INPUT_TYPE_MULTIMEDIA) ||
		input_type == INPUT_TYPE_ERROR)
		{
			printf("Wrong input type. Closing connection.\n");
			server_send_status(STATUS_ERROR);
			break;
		}

		input_open_write(input_type);

		server_send_status(STATUS_OK);



__s32 event_light_value_0=0;
__s32 event_light_value_1=0;

struct input_event_light event_light;
struct input_event event;

int trick=0;

switch(input_type)
{
	case INPUT_TYPE_TOUCH:
	case INPUT_TYPE_TOUCH_ROTATE:
		while(read(sock_client_fd, &event_light, sizeof(struct input_event_light)))
		{
			if(event_light.type == 3)
			{
				event.type=EV_REL;
		
				if(event_light.code == 0)
				{
					if(!event_light_value_0)
						event_light_value_0=event_light.value;

					if(input_type == INPUT_TYPE_TOUCH_ROTATE)
						event.code=REL_X;
					else
						event.code=REL_Y;

					event.value=event_light_value_0 - event_light.value;
					event.value*=speed_factor;

					event_light_value_0=event_light.value;

					input_event_write(event);

				}
				else if(event_light.code == 1)
				{
					if(!event_light_value_1)
						event_light_value_1=event_light.value;

					event.value=event_light_value_1 - event_light.value;

					if(input_type == INPUT_TYPE_TOUCH_ROTATE)
						event.code=REL_Y;
					else
					{
						event.code=REL_X;
						event.value*=-1;

					}

					event.value*=speed_factor;

					event_light_value_1=event_light.value;

					input_event_write(event);
				}
				/*
				else if(event_light.code == 24 && event_light.value == 1)
				{
					down_tap.pos0=event_light_value_0;
					down_tap.pos1=event_light_value_1;
				}
				*/
				else if(event_light.code == 24 && event_light.value == 0)
				{
					/*
					event.type=EV_KEY;
					event.code = BTN_LEFT;
					event.value = 0;

					input_event_write(event);
					*/

					event_light_value_0=0;
					event_light_value_1=0;


				}
				
			}
			if(event_light.type == 1 && event_light.code == 169)
			{
				if(event_light.value == 1)
				{
					event.type=EV_KEY;
					event.code = BTN_LEFT;
					event.value = 1;

					input_event_write(event);
				}
				else if(event_light.value == 0)
				{
					event.type=EV_KEY;
					event.code = BTN_LEFT;
					event.value = 0;

					input_event_write(event);
				}
			}
			if(event_light.type == 1 && event_light.code == 116)
			{
				if(event_light.value == 1)
				{
					event.type=EV_KEY;
					event.code = BTN_RIGHT;
					event.value = 1;

					input_event_write(event);
				}
				else if(event_light.value == 0)
				{
					event.type=EV_KEY;
					event.code = BTN_RIGHT;
					event.value = 0;

					input_event_write(event);
				}
			}
			if(event_light.type == 0 && event_light.code == 0 && event_light.value == 0)
			{
				event.type = EV_SYN;
				event.code = SYN_REPORT;
				event.value = 0;

				input_event_write(event);
			}
		}
	break;
	case INPUT_TYPE_ACCEL:
		/*
		75 -> -75  : rien
		0 -> 750   : petite inclinaison gauche -> KEY_RIGHTSHIFT
		0 -> 1000  : grosse inclinaison gauche
		0 -> -750  : petite inclinaison droite -> KEY_RIGHTSHIFT
		0 -> -1000 : grosse inclinaison droite

		0 - 750    : par accoups
		750 - * : en continu
		*/

		while(read(sock_client_fd, &event_light, sizeof(struct input_event_light)))
		{
			if(event_light.type == 3)
			{
				event.type=EV_KEY;

				if(event_light.code == 1 && (event_light.value > 100 || event_light.value < -100))
				{
					if(event_light.value > 0 && event_light.value > 600)
					{
						event.code=KEY_LEFT;
						event.value=1;
						input_event_write(event);
						event.value=0;
						input_event_write(event);
					}
					else if(event_light.value > 0 && event_light.value < 600)
					{
						event.code=KEY_RIGHTSHIFT;
						event.value=1;
						input_event_write(event);

						event.code=KEY_LEFT;
						event.value=1;
						input_event_write(event);

						event.code=KEY_LEFT;
						event.value=0;
						input_event_write(event);

						event.code=KEY_RIGHTSHIFT;
						event.value=0;
						input_event_write(event);
					}
					
					if(event_light.value < 0 && event_light.value < -600)
					{
						event.code=KEY_RIGHT;
						event.value=1;
						input_event_write(event);
						event.value=0;
						input_event_write(event);
					}
					else if(event_light.value < 0 && event_light.value > -600)
					{
						event.code=KEY_RIGHTSHIFT;
						event.value=1;
						input_event_write(event);

						event.code=KEY_RIGHT;
						event.value=1;
						input_event_write(event);

						event.code=KEY_RIGHT;
						event.value=0;
						input_event_write(event);

						event.code=KEY_RIGHTSHIFT;
						event.value=0;
						input_event_write(event);
					}
				}
			}
			if(event_light.type == 1)
			{
				event.type=EV_KEY;

				if(event_light.code == 169)
				{
					if(event_light.value == 1)
					{
						event.code=KEY_DOWN;
						event.value=1;
						input_event_write(event);
					}
					else if(event_light.value == 0)
					{
						event.code=KEY_DOWN;
						event.value=0;
						input_event_write(event);
					}
				}
				if(event_light.code == 116)
				{
					if(event_light.value == 1)
					{
						event.code=KEY_UP;
						event.value=1;
						input_event_write(event);
					}
					else if(event_light.value == 0)
					{
						event.code=KEY_UP;
						event.value=0;
						input_event_write(event);
					}
				}
			}

			if(event_light.type == 0 && event_light.code == 0 && event_light.value == 0)
			{
				event.type = EV_SYN;
				event.code = SYN_REPORT;
				event.value = 0;

				input_event_write(event);
			}
		}
	break;
}

	printf("Connection closed\n");

close(sock_client_fd);
}

}

int server_get_input_type(void)
{
	int input_type=INPUT_TYPE_ERROR;

	printf("Getting input type...\n");

	if(!read(sock_client_fd, &input_type, sizeof(input_type)))
	{
		perror("Error in write()");
		exit(1);
	}

	return input_type;
}

void server_send_status(int status)
{
	printf("Sending status...\n");

	if(!write(sock_client_fd, &status, sizeof(status)))
	{
		perror("Error in write()");
		exit(1);
	}
}

void server_close(void)
{
	close(sock_fd);
}
